// Copyright 2006-2007 all rights reserved; see LICENSE file for BSD-style license

package edu.berkeley.sbp;
import edu.berkeley.sbp.Sequence.Pos;
import edu.berkeley.sbp.Sequence.Pos;

final class Reduction implements Comparable<Reduction> {

    private Pos reduction;
    private GSS.Phase phase;
    private Forest forest;
    private Node parent;
    
    public Reduction(Node parent, Pos reduction, Forest forest, GSS.Phase target) {
        this.reduction = reduction;
        this.forest = forest;
        this.phase = target;
        this.parent = parent;
        target.addReduction(this);
    }

    public int compareTo(Reduction r) {
        if (parent.phase()!=null || r.parent.phase()!=null) {
            if (parent.phase()==null) return 1;
            if (r.parent.phase()==null) return -1;
            if (parent.phase().pos < r.parent.phase().pos) return 1;
            if (parent.phase().pos > r.parent.phase().pos) return -1;
        }
        /*
        int master = Parser.mastercache.comparePositions(reduction(), r.reduction());
        if (master != 0 && master != signum(reduction.ord() - r.reduction.ord()))
            throw new Error("master="+master+" and a="+reduction.ord()+" and b="+r.reduction.ord()+"\n"+reduction+"\n"+r.reduction);
        */
        return reduction.compareTo(r.reduction);
    }

    private int signum(int x) {
        if (x==0) return 0;
        if (x<0) return -1;
        return 1;
    }

    public void perform() { new Result(forest, parent, reduction, phase); }
    public GSS.Phase parentPhase() { return parent.phase(); }
    public Pos reduction() { return reduction; }
    public GSS.Phase targetPhase() { return phase; }
    public String toString() { return (parent.phase()==null ? 0 : parent.phase().pos) + ":"+reduction; }
}
